# Emmy version 1.0.0
# EmmyDDNode.py
# by Antoine Rauzy
# Copyrights (c) 2019 AltaRica Association
#
# This file is part of Emmy, a pedagogical stochastic simulator.
# Emmy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Emmy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Emmy.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creators
# 3) Access methods

# 1) Imported modules
# -------------------

from S2MLCore import *

# 2) Creators
# -----------

class EmmyDDNode:
  LEAF = 1
  NODE = 2

  def __init__(self, number, type):
    self._number = number
    self._type = type
    self._nextInCollisionChain = None
    self._flag = 0

  @staticmethod
  def NewLeaf(number, constant):
    leaf = EmmyDDNode(number, EmmyDDNode.LEAF)
    leaf._constant = constant
    return leaf

  @staticmethod
  def NewNode(number, index, thenChild, elseChild):
    node = EmmyDDNode(number, EmmyDDNode.NODE)
    node._index = index
    node._thenChild = thenChild
    node._elseChild = elseChild
    return node

# 3) Access methods
# -----------------

  def GetNumber(self):
    return self._number

  def GetType(self):
    return self._type

  def IsLeaf(self):
    return self._type==EmmyDDNode.LEAF

  def IsNode(self):
    return self._type==EmmyDDNode.NODE

  def GetConstant(self):
    if not self.IsLeaf():
      return None
    return self._constant

  def GetIndex(self):
    if not self.IsNode():
      return -1
    return self._index

  def GetThenChild(self):
    if not self.IsNode():
      return None
    return self._thenChild

  def GetElseChild(self):
    if not self.IsNode():
      return None
    return self._elseChild

  def GetNextInCollisionChain(self):
    return self._nextInCollisionChain

  def SetNextInCollisionChain(self, node):
    self._nextInCollisionChain = node

  def GetFlag(self):
    return self._flag

  def SetFlag(self, value):
    self._flag = value

  def ResetFlag(self):
    self.SetFlag(0)

  def SetFlags(self, value):
    if self._flag==value:
      return
    self._flag = value
    if self._type==EmmyDDNode.LEAF:
      return
    self._thenChild.SetFlags(value)
    self._elseChild.SetFlags(value)

  def ResetFlags(self):
    self.SetFlags(0)
