# Emmy version 1.0.0
# EmmyDDUniqueTable.py
# by Antoine Rauzy
# Copyrights (c) 2019 AltaRica Association
#
# This file is part of Emmy, a pedagogical stochastic simulator.
# Emmy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Emmy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Emmy.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creators
# 3) Nodes

# 1) Imported modules
# -------------------

from S2MLCore import *

from EmmyDDNode import *

# 2) Creators
# -----------

class EmmyDDUniqueTable:
  def __init__(self, size):
    self._size = size
    self._hashTable = [None for i in range(0, self._size)]

  def Reset(self):
    self._hashTable = [None for i in range(0, self._size)]
    
# 3) Nodes
# --------

  def LookForNode(self, index, thenChild, elseChild):
    hashCode = self.ComputeHashCode(index, thenChild, elseChild)
    node = self._hashTable[hashCode]
    while node!=None:
      if node.GetIndex()==index and node.GetThenChild()==thenChild and node.GetElseChild()==elseChild:
        break
      node = node.GetNextInCollisionChain()
    return node

  def NewNode(self, number, index, thenChild, elseChild):
    node = EmmyDDNode.NewNode(number, index, thenChild, elseChild)
    hashCode = self.ComputeHashCode(index, thenChild, elseChild)
    node.SetNextInCollisionChain(self._hashTable[hashCode])
    self._hashTable[hashCode] = node
    return node

  def ComputeHashCode(self, index, thenChild, elseChild):
    return (7*index + 13*thenChild.GetNumber() + 17*elseChild.GetNumber()) % self._size