# Emmy version 1.0.0
# EmmyFDSConstant.py
# by Antoine Rauzy
# Copyrights (c) 2019 AltaRica Association
#
# This file is part of Emmy, a pedagogical stochastic simulator.
# Emmy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Emmy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Emmy.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creators
# 3) Access methods
# 4) Predecessors
# 5) Order

# 1) Imported modules
# -------------------

from S2MLCore import *

from EmmyDecisionDiagrams import *

# 2) Creators
# -----------

class EmmyFDSConstant:
  def __init__(self, constant):
    self._constant = constant
    self._fdsPredecessors = []
    
# 3) Access methods
# -----------------

  def GetConstant(self):
    return self._constant

  def IsEq(self, other):
    return S2MLConstant.IsEq(self._constant, other._constant)

# 4) Predecessors
# ---------------

  def GetFDSPredecessors(self):
    return self._fdsPredecessors

  def IsFDSPredecessor(self, fdsConstant):
    for fdsCandidate in fdsConstant._fdsPredecessors:
      if fdsCandidate.IsEq(self):
        return True
    return False

  def IsFDSSuccessor(self, fdsConstant):
    return fdsConstant.IsFDSPredecessor(self)

  def AppendFDSPredecessor(self, fdsConstant):
  # REQUIRES: not self.IsSmaller(fdsConstant)
    self._fdsPredecessors.append(fdsConstant)
 
# 5) Order
# --------

  def IsFDSBigger(self, fdsConstant):
    for fdsPredecessor in self._fdsPredecessors:
      if fdsPredecessor.IsEq(fdsConstant) or fdsPredecessor.IsFDSBigger(fdsConstant):
        return True
    return False

  def IsFDSSmaller(self, fdsConstant):
    return fdsConstant.IsFDSBigger(self)

  def IsFDSBiggerOrEqual(self, fdsConstant):
    return self.IsEq(fdsConstant) or self.IsFDSBigger(fdsConstant)

  def IsFDSSmallerOrEqual(self, fdsConstant):
    return self.IsEq(fdsConstant) or self.IsFDSSmaller(fdsConstant)

  def IsFDSLeastElement(self):
    return len(self._fdsPredecessors)==0