# Emmy version 1.0.0
# EmmyFDSLiteral.py
# by Antoine Rauzy
# Copyrights (c) 2019 AltaRica Association
#
# This file is part of Emmy, a pedagogical stochastic simulator.
# Emmy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Emmy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Emmy.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creators
# 3) Access methods
# 4) Predecessors
# 5) Order

# 1) Imported modules
# -------------------

from S2MLCore import *

from EmmyDecisionDiagrams import *
from EmmyFDSVariable import *
from EmmyFDSConstant import *

# 2) Creators
# -----------

class EmmyFDSLiteral:
  UNKNOWN = 0
  FIRST = 1
  INTERMEDIATE = 2
  LAST = 3
  def __init__(self, fdsVariable, fdsConstant):
    self._fdsVariable = fdsVariable
    self._fdsConstant = fdsConstant
    self._position = EmmyFDSLiteral.UNKNOWN
    self._ddIndex = -1
    self._probabilityDistribution = None
    self._probability = 0.0

# 3) Access methods
# -----------------

  def GetFDSVariable(self):
    return self._fdsVariable

  def GetFDSConstant(self):
    return self._fdsConstant

  def GetPosition(self):
    return self._position

  def SetPosition(self, position):
    self._position = position

  def GetDDIndex(self):
    return self._ddIndex

  def SetDDIndex(self, index):
    self._ddIndex = index

  def GetVariable(self):
    return self._fdsVariable.GetVariable()

  def GetConstant(self):
    return self._fdsConstant.GetConstant()

  def GetProbabilityDistribution(self):
    return self._probabilityDistribution

  def SetProbabilityDistribution(self, probabilityDistribution):
    self._probabilityDistribution = probabilityDistribution

  def GetProbability(self):
    return self._probability

  def SetProbability(self, probability):
    self._probability = probability

