# Emmy version 1.0.0
# EmmyModel.py
# by Antoine Rauzy
# Copyrights (c) 2019 AltaRica Association
#
# This file is part of Emmy, a pedagogical stochastic simulator.
# Emmy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Emmy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Emmy.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creator
# 3) Methods to test types
# 4) Factory of elements
# 5) Methods for the parser
# 6) Methods for the checker

# 1) Imported modules
# -------------------

from S2MLCore import *

from EmmyFiniteDegradationStructures import *

from EmmyType import *
from EmmyDomainDeclaration import *
from EmmyBlock import *
from EmmyParameter import *
from EmmyVariable import *
from EmmyProbabilityDeclaration import *
from EmmyAssignment import *
from EmmyObserver import *

# 2) Creator
# ---------

class EmmyModel(S2MLModel):

  def __init__(self):
    S2MLModel.__init__(self)

# 3) Methods to test types
# ------------------------

  @staticmethod
  def IsVariable(element):
    return element.GetType()==EmmyType.VARIABLE

  @staticmethod
  def IsAssignment(element):
    return element.GetType()==EmmyType.ASSIGNMENT

  @staticmethod
  def IsObserver(element):
    return element.GetType()==EmmyType.OBSERVER

# 4) Factory of elements
# ----------------------

  def NewDomainDeclaration(self, identifier, domain, container):
    return EmmyDomainDeclaration(identifier, domain, container)

  def NewBlock(self, identifier, container):
    return EmmyBlock(identifier, container)

  def NewVariable(self, domain, path, container):
    return EmmyVariable(domain, path, container)

  def NewParameter(self, domain, path, container):
    return EmmyParameter(domain, path, container)

  def NewProbabilityDeclaration(self, variablePath, constant, distribution, container):
    return EmmyProbabilityDeclaration(variablePath, constant, distribution, container)

  def NewAssignment(self, variablePath, expression, container):
    return EmmyAssignment(variablePath, expression, container)

  def NewObserver(self, path, definition, container):
    return EmmyObserver(self.NewRealDomain(), path, definition, container)

# 5) Methods for the parser
# -------------------------

  def CanComposeVariable(self, container):
    return S2MLModel.IsBlock(container) or S2MLModel.IsClass(container)

  def CanComposeAssignment(self, container):
    return S2MLModel.IsBlock(container) or S2MLModel.IsClass(container)

  def CanComposeProbabilityDeclaration(self, container):
    return S2MLModel.IsBlock(container) or S2MLModel.IsClass(container)

  def CanComposeObserver(self, container):
    return S2MLModel.IsBlock(container) or S2MLModel.IsClass(container)

# 6) Methods for the checker
# --------------------------

  def IsValidReferredElementInStochasticExpression(self, element):
    return element.GetType()==EmmyType.PARAMETER or element.GetType()==EmmyType.VARIABLE

