# Emmy version 1.1.0
# EmmyVariable.py
# by Antoine Rauzy
# Copyrights (c) 2018 AltaRica Association
#
# This file is part of Emmy, a pedagogical stochastic simulator.
# Emmy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Emmy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Emmy.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creator
# 3) Access methods

# 1) Imported modules
# -------------------

from S2MLCore import *

from EmmyType import *

# 2) Creator
# ----------

class EmmyVariable(S2MLPort):
  UNKNOWN = 0
  STATE = 1
  FLOW = 2

  def __init__(self, domain, path, container):
    S2MLPort.__init__(self, EmmyType.VARIABLE, domain, path, container)
    if path.IsIdentifier():
      self.Rename(path.ToString())
    self._category = EmmyVariable.UNKNOWN
    self._index = 0
    self._value = S2MLConstant.NewNone()
    self._fdsVariable = None

# 3) Access methods
# -----------------

  def GetCategory(self):
    return self._category
  def SetCategory(self, category):
    self._category = category
  def IsStateVariable(self):
    return self._category==EmmyVariable.STATE
  def IsFlowVariable(self):
    return self._category==EmmyVariable.FLOW

  def GetIndex(self):
    return self._index
  def SetIndex(self, index):
    self._index = index

  def IsValueCalculated(self):
    return not self._value.IsNone()
  def ResetValue(self):
    self._value.Reset()
  def GetValue(self):
    return self._value
  def SetValue(self, value):
    self._value.Set(value)

  def GetFDSVariable(self):
    return self._fdsVariable
  def SetFDSVariable(self, fdsVariable):
    self._fdsVariable = fdsVariable

