# S2MLCore version 1.0.0
# S2MLContainer.py
# by Antoine Rauzy
# Copyrights (c) 2018 AltaRica Association
#
# This file is part of S2MLCore.
# S2MLCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# S2MLCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with S2MLCore.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Containers

# 1) Imported modules
# -------------------

from S2MLPath import *
from S2MLElement import *

# 2) Containers
# -------------

class S2MLContainer(S2MLElement):
  def __init__(self, type, container):
    S2MLElement.__init__(self, type, container)
    self._isAtomic = False
    self._elements = []
    self._dictionary = dict()
    self._aliases = dict()

  def DeleteAll(self):
    while self.GetNumberOfElements()>0:
      element = self.GetElement(0)
      element.Delete()
    self.Delete()

  def GetNumberOfElements(self):
    return len(self._elements)
  def GetElement(self, index):
    if index<0 or index>=self.GetNumberOfElements():
      return None
    return self._elements[index]
  def GetElements(self):
    return self._elements
  def GetDictonary(self):
    return self._dictionary
  def LookForIdentifiedElement(self, identifier, checkType = None):
    element = self._dictionary.get(identifier, None)
    if element==None:
      return None
    if checkType!=None and not checkType(element):
      return None
    return element
  def _AddElement(self, element):
    self._elements.append(element)
  def _RemoveElement(self, element):
    self._elements.remove(element)
  def _AddToDictonary(self, element):
    # REQUIRE: self.LookForIdentifiedElement(element.GetIdentifier())==None
    self._dictionary[element.GetIdentifier()] = element
  def _RemoveFromDictonary(self, element):
    del self._dictionary[element.GetIdentifier()]
  def AccessNamedElement(self, path):
    if path.IsEmpty():
      return self
    identifier = path.RemoveFirstIdentifier()
    if identifier==S2MLPath.ROOT_IDENTIFIER:
      container = self.GetRootContainer()
      element = container.AccessNamedElement(path)
    elif identifier==S2MLPath.CONTAINER_IDENTIFIER:
      container = self.GetContainer()
      if container==None:
        element = None
      else:
        element = container.AccessNamedElement(path)
    else:
      aliasPath = self.GetAlias(identifier)
      if aliasPath==None:
        element = self.LookForIdentifiedElement(identifier)
        if element!=None:
          if not element.IsAtomic():
            container = element
            element = container.AccessNamedElement(path)
          elif not path.IsEmpty():
            element = None
      else:
        aliasedPath = S2MLPath.Merge(aliasPath, path)
        element = self.AccessNamedElement(aliasedPath)
        if element==None:
          print("Nope")
    path.PrependIdentifier(identifier)
    return element
  def RemoveElement(self, element):
    if not element in self._elements:
      return -1
    index = self._elements.index(element)
    self._elements.remove(element)
    if element.GetIdentifier()!=None:
      self._RemoveFromDictonary(element)
    element._container = None
    return index
  def AddElement(self, element):
    self._elements.append(element)
    if element.GetIdentifier()!=None:
      self._AddToDictonary(element)
    element._container = self
  def InsertElement(self, index, element):
    self._elements.insert(index, element)
    if element.GetIdentifier()!=None:
      self._AddToDictonary(element)
    element._container = self

  def CollectElements(self, test, recursive, elements):
    for element in self.GetElements():
      if test(element):
        elements.append(element)
      if not element._isAtomic and recursive:
        element.CollectElements(test, recursive, elements)
    if test(self):
      elements.append(self)

  def SortElements(self, getKeyOfElement):
    self._elements.sort(key = getKeyOfElement)

  def GetAlias(self, identifier):
    return self._aliases.get(identifier, None)
  def SetAlias(self, identifier, path):
    self._aliases[identifier] = path

  def SetFlagsOfElements(self, value):
    for element in self._elements:
      element.SetFlag(value)
      if not element.IsAtomic():
        element.SetFlagsOfElements(value)
  def ResetFlagsOfElements(self):
    self.SetFlagsOfElements(0)

  def SetAttributesOfElements(self, testElement, attributeName, attributeValue, recursive=True):
    for element in self.GetElements():
      if testElement(element):
        element.SetAttribute(attributeName, attributeValue)
      if not element.IsAtomic() and recursive:
        element.SetAttributesOfElements(testElement, attributeName, attributeValue, True)
  def ResetAttributesOfElements(self, testElement, attributeName, recursive=True):
    for element in self.GetElements():
      if testElement(element):
        element.ResetAttribute(attributeName)
      if not element.IsAtomic() and recursive:
        element.ResetAttributesOfElements(testElement, attributeName, True)