# S2MLCore version 1.0.0
# S2MLElement.py
# by Antoine Rauzy
# Copyrights (c) 2018 AltaRica Association
#
# This file is part of S2MLCore.
# S2MLCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# S2MLCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with S2MLCore.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creator
# 3) Access methods
# 4) Attributes

# 1) Imported modules
# -------------------

from S2MLPath import *

# 2) Creator
# ---------

class S2MLElement:
  def __init__(self, type, container):
    self._type = type
    self._isAtomic = True
    self._identifier = None
    self._path = None
    self._container = container
    if container!=None:
      container._AddElement(self)
    self._definition = None
    self._flag = 0
    self._attributes = dict()

  def Delete(self):
    if self._container==None:
      return
    if self._identifier!=None:
      self._container._RemoveFromDictonary(self)
    self._container._RemoveElement(self)

# 3) Access methods
# -----------------

  def GetType(self):
    return self._type
  def SetType(self, type):
    self._type = type

  def IsAtomic(self):
    return self._isAtomic

  def GetIdentifier(self):
    return self._identifier
  def SetIdentifier(self, identifier):
    self._identifier = identifier
  def Rename(self, identifier):
    if self._container==None:
      self._identifier = identifier
      return
    if self._identifier!=None:
      self._container._RemoveFromDictonary(self)
    self._identifier = identifier
    self._container._AddToDictonary(self)

  def GetPath(self):
    return self._path
  def SetPath(self, path):
    self._path = path

  def GetAbsolutePath(self):
    path = S2MLPath.Copy(self.GetPath())
    container = self.GetContainer()
    if container!=None:
      while container.GetContainer()!=None:
        path.PrependIdentifier(container.GetIdentifier())
        container = container.GetContainer()
    return path

  def GetContextualizedAbsolutePath(self):
    path = self.GetAbsolutePath()
    path.RemoveFirstIdentifier()
    path.PrependIdentifier(S2MLPath.ROOT_IDENTIFIER)
    return path

  def GetContainer(self):
    return self._container
  def GetModel(self):
    if self.GetContainer()==None:
      return self
    return self.GetContainer().GetModel()
  def GetRootContainer(self):
    container = self.GetContainer()
    if container==None:
      return None
    parentContainer = container.GetContainer()
    if parentContainer==None:
      return self
    return container.GetRootContainer()

  def GetDefinition(self):
    return self._definition
  def SetDefinition(self, definition):
    self._definition = definition

  def GetFlag(self):
    return self._flag
  def SetFlag(self, value):
    self._flag = value

# 4) Attributes
# -------------

  def GetNumberOfAttributes(self):
    return len(self._attributes)

  def GetAttribute(self, name, defaultValue = None):
    return self._attributes.get(name, defaultValue)

  def SetAttribute(self, name, value):
    self._attributes[name] = value

  def ResetAttribute(self, name):
    self._attributes.pop(name, None)

  def ResetAttributes(self):
    self._attributes.clear()

  def GetAttributeNames(self):
    return self._attributes
