# S2MLCore version 1.0.0
# S2MLBin.py
# by Antoine Rauzy
# Copyrights (c) 2018 AltaRica Association
#
# This file is part of S2MLCore.
# S2MLCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# S2MLCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with S2MLCore.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Bins

# 1) Imported modules
# -------------------

from S2MLDomains import *

# 2) Bins
# -------

class S2MLBin:
  def __init__(self):
    self._sumOfValues = 0.0
    self._sumOfSquareOfValues = 0.0
    self._numberOfValues = 0
    self._minimumValue = math.inf
    self._maximumValue = -math.inf

  def GetSumOfValues(self):
    return self._sumOfValues
  def GetSumOfSquareOfValues(self):
    return self._sumOfSquareOfValues
  def GetNumberOfValues(self):
    return self._numberOfValues
  def GetMinimumValue(self):
    return self._minimumValue
  def GetMaximumValue(self):
    return self._maximumValue

  def Reset(self):
    self._sumOfValues = 0.0
    self._sumOfSquareOfValues = 0.0
    self._numberOfValues = 0
    self._minimumValue = math.inf
    self._maximumValue = -math.inf

  def AddValue(self, value):
    self._sumOfValues += value
    self._sumOfSquareOfValues += value*value
    self._numberOfValues += 1
    if value<self._minimumValue:
      self._minimumValue = value
    if value>self._maximumValue:
      self._maximumValue = value

  def GetMean(self):
    if self._numberOfValues==0:
      return 0.0
    return self._sumOfValues/self._numberOfValues
  def GetVariance(self):
    if self._numberOfValues<=1:
      return 0.0
    mean = self.GetMean()
    variance = (self._sumOfSquareOfValues - mean*mean*self._numberOfValues)/(self._numberOfValues-1)
    if variance<0:
      variance = -variance
    return variance
  def GetStandardDeviation(self):
    return math.sqrt(self.GetVariance())

  def _GetErrorFactor(self, factor):
    if self._numberOfValues<=1:
      return 0.0
    return (factor*self.GetStandardDeviation())/math.sqrt(self._numberOfValues)
  def GetErrorFactor90(self):
    return self._GetErrorFactor(1.64)
  def GetErrorFactor95(self):
    return self._GetErrorFactor(1.96)
  def GetErrorFactor99(self):
    return self._GetErrorFactor(2.58)
